@@ -1,48 +1,60 @@
 import re
 from typing import Dict, List
 from simple_ddl_parser.parser import Parser
-
+from simple_ddl_parser.dialects.hql import HQL
 
 _ref = "REFERENCES"
 _def = "DEFAULT"
 _cons = "CONSTRAINT"
 
 
-class DDLParser(Parser):
+class DDLParser(Parser, HQL):
     """
     lex and yacc parser for parse ddl into BQ schemas
     """
 
-    reserved = {
+    defenition_statements = {
         "IF": "IF",
-        "THEN": "THEN",
-        "ON": "ON",
-        "ELSE": "ELSE",
-        "WHILE": "WHILE",
-        "USE": "USE",
+        "NOT": "NOT",
+        "DROP": "DROP",
         "CREATE": "CREATE",
         "TABLE": "TABLE",
-        "DROP": "DROP",
-        "NOT": "NOT",
         "EXISTS": "EXISTS",
-        "NULL": "NULL",
-        "NUM_VALUE_SDP": "NUM_VALUE_SDP",
-        "PRIMARY": "PRIMARY",
-        "KEY": "KEY",
-        "DEFAULT": "DEFAULT",
-        "REFERENCES": "REFERENCES",
         "ALTER": "ALTER",
         "ADD": "ADD",
-        "FOREIGN": "FOREIGN",
-        "UNIQUE": "UNIQUE",
         "CHECK": "CHECK",
-        "SEQUENCE": "SEQUENCE",
         "CONSTRAINT": "CONSTRAINT",
+        "FOREIGN": "FOREIGN",
+        "INDEX": "INDEX",
+        "SEQUENCE": "SEQUENCE",
+        "REFERENCES": "REFERENCES",
+        "KEY": "KEY",
+    }
+
+    columns_defenition = {
+        "ON": "ON",
+        "NULL": "NULL",
+        "PRIMARY": "PRIMARY",
+        "DEFAULT": "DEFAULT",
+        "UNIQUE": "UNIQUE",
         "ARRAY": "ARRAY",
-        "INDEX": "INDEX"
+        ",": "COMMA",
+    }
+    after_columns_tokens = {
+        "PARTITIONED": "PARTITIONED",
+        "BY": "BY",
+        "STORED": "STORED",
+        "AS": "AS",
+        "LOCATION": "LOCATION",
+        "ROW": "ROW",
+        "FORMAT": "FORMAT",
+        "FIELDS": "FIELDS",
+        "TERMINATED": "TERMINATED",
+        "COLLECTION": "COLLECTION",
+        "ITEMS": "ITEMS",
+        "MAP": "MAP",
+        "KEYS": "KEYS",
     }
-    
-    sequence = False
     sequence_reserved = {
         "INCREMENT": "INCREMENT",
         "START": "START",
@@ -51,33 +63,64 @@ class DDLParser(Parser):
         "CACHE": "CACHE",
     }
     tokens = tuple(
-        ["ID", "NEWLINE", "DOT"]
-        + list(reserved.values())
+        ["ID", "NEWLINE", "DOT", "STRING", "LP", "RP"]
+        + list(defenition_statements.values())
+        + list(columns_defenition.values())
         + list(sequence_reserved.values())
+        + list(after_columns_tokens.values())
     )
 
-    t_ignore = '\t();,  "\r'
+    t_ignore = ';\t  "\r'
     t_DOT = r"."
 
-    def t_NUM_VALUE_SDP(self, t):
-        r"[0-9]+\D"
-        t.type = "NUM_VALUE_SDP"
-        t.value = re.sub(r"[\)\,;]", "", t.value)
+    sequence = False
+    last_token = False
+    columns_def = False
+    after_columns = False
+
+    def t_STRING(self, t):
+        r"\'[a-zA-Z_,0-9:><\=\-\+\~\%$'\!(){}\[\]\/\\\"]*\'\B"
+        t.type = "STRING"
         return t
 
     def t_ID(self, t):
-        r"[a-zA-Z_0-9:><\=\-\+\~\%$\'!{}\[\]]+"
-
-        if self.sequence:
-            t.type = self.sequence_reserved.get(t.value.upper(), "ID")
-        elif 'ARRAY' in t.value:
-            t.type = 'ARRAY'
+        r"[a-zA-Z_,0-9:><\/\=\-\+\~\%$'\()!{}\[\]]+"
+        if t.value == ")":
+            t.type = "RP"
+        elif t.value == "(":
+            t.type = "LP"
+            if not self.after_columns:
+                self.columns_def = True
         else:
-            t.type = self.reserved.get(t.value.upper(), "ID")  # Check for reserved word
+            t.type = self.defenition_statements.get(
+                t.value.upper(), "ID"
+            )  # Check for reserved word
+        if t.value.strip() == "'":
+            self.string = True
+        if t.type == "CREATE":
+            self.sequence = False
+            self.is_table = False
+        if self.last_token == "RP" or self.after_columns:
+            t.type = self.after_columns_tokens.get(t.value.upper(), t.type)
+            if t.type != "ID":
+                self.after_columns = True
+            elif self.columns_def:
+                t.type = self.columns_defenition.get(t.value.upper(), t.type)
+        elif self.columns_def:
+            t.type = self.columns_defenition.get(t.value.upper(), t.type)
+        elif self.sequence:
+            t.type = self.sequence_reserved.get(t.value.upper(), "ID")
+        elif "ARRAY" in t.value:
+            t.type = "ARRAY"
+        if t.type == "TABLE" or t.type == "INDEX":
+            self.is_table = True
+        elif t.type == "SEQUENCE" and self.__dict__.get("is_table"):
+            t.type = "ID"
         if t.type == "SEQUENCE":
             self.sequence = True
         if t.type != "ID":
             t.value = t.value.upper()
+        self.last_token = t.type
         return t
 
     def t_newline(self, t):
@@ -93,6 +136,12 @@ class DDLParser(Parser):
     def p_error(self, p):
         pass
 
+    def p_expression_partitioned_by(self, p):
+        """expr : expr PARTITIONED BY LP pid RP"""
+        p[0] = p[1]
+        p_list = list(p)
+        p[0]["partitioned_by"] = p_list[-2]
+
     def p_expression_drop_table(self, p):
         """expr : DROP TABLE ID
         | DROP TABLE ID DOT ID
@@ -107,21 +156,20 @@ class DDLParser(Parser):
         else:
             table_name = p_list[-1]
         p[0] = {"schema": schema, "table_name": table_name}
-    
+
     def p_expression_index(self, p):
-        """expr : index_table_name ID
-                | expr ID
-        """
-        p_list = list(p)
+        """expr : index_table_name LP pid RP"""
+        p_list = remove_par(list(p))
         p[0] = p[1]
-        if not 'columns' in p[0]:
-            p[0]['columns'] = [p_list[-1]]
+
+        if not "columns" in p[0]:
+            p[0]["columns"] = p_list[-1]
         else:
-            p[0]['columns'].append(p_list[-1])
-    
+            p[0]["columns"].append(p_list[-1])
+
     def p_index_table_name(self, p):
-        """index_table_name : create_index ON ID  
-                | create_index ON ID DOT ID  
+        """index_table_name : create_index ON ID
+        | create_index ON ID DOT ID
         """
         p[0] = p[1]
         p_list = list(p)
@@ -132,25 +180,57 @@ class DDLParser(Parser):
         else:
             table_name = p_list[-1]
         p[0].update({"schema": schema, "table_name": table_name})
-        
-        
+
     def p_create_index(self, p):
         """create_index : CREATE INDEX ID
-                        | CREATE UNIQUE INDEX ID
-                        | create_index ON ID
+        | CREATE UNIQUE INDEX ID
+        | create_index ON ID
         """
         p_list = list(p)
         if isinstance(p[1], dict):
             p[0] = p[1]
         else:
-            p[0] = {"schema": None, "index_name": p_list[-1], "unique": 'UNIQUE' in p_list}
-    
-    def p_expression_table_name(self, p):
-        """expr : create_table ID DOT ID
+            p[0] = {
+                "schema": None,
+                "index_name": p_list[-1],
+                "unique": "UNIQUE" in p_list,
+            }
+
+    def p_expression_table(self, p):
+        """expr : table_name defcolumn
+        | table_name LP defcolumn
+        | expr COMMA defcolumn
+        | expr COMMA
+        | expr COMMA constraint
+        | expr COMMA check_ex
+        | expr COMMA foreign
+        | expr COMMA pkey
+        | expr COMMA uniq
+        | expr RP
+        """
+        p[0] = p[1]
+        p_list = list(p)
+        if p_list[-1] != "," and p_list[-1] != ")":
+            if "type" in p_list[-1] and "name" in p_list[-1]:
+                p[0]["columns"].append(p_list[-1])
+            elif "check" in p_list[-1]:
+                if isinstance(p_list[-1]["check"], list):
+                    check = " ".join(p_list[-1]["check"])
+                    if isinstance(check, str):
+                        check = {"constraint_name": None, "statement": check}
+                else:
+                    check = p_list[-1]["check"]
+                p[0]["checks"].append(check)
+            else:
+                p[0].update(p_list[-1])
+
+    def p_table_name(self, p):
+        """table_name : create_table ID DOT ID
         | create_table ID
         """
         # get schema & table name
         p_list = list(p)
+        p[0] = p[1]
         schema = None
         if len(p) > 4:
             if "." in p:
@@ -158,15 +238,17 @@ class DDLParser(Parser):
                 table_name = p_list[-1]
         else:
             table_name = p_list[-1]
-        p[0] = {"schema": schema, "table_name": table_name}
+        p[0].update(
+            {"schema": schema, "table_name": table_name, "columns": [], "checks": []}
+        )
 
     def p_expression_seq(self, p):
         """expr : seq_name
-        | expr INCREMENT NUM_VALUE_SDP
-        | expr START NUM_VALUE_SDP
-        | expr MINVALUE NUM_VALUE_SDP
-        | expr MAXVALUE NUM_VALUE_SDP
-        | expr CACHE NUM_VALUE_SDP
+        | expr INCREMENT ID
+        | expr START ID
+        | expr MINVALUE ID
+        | expr MAXVALUE ID
+        | expr CACHE ID
         """
         # get schema & table name
         p_list = list(p)
@@ -200,17 +282,23 @@ class DDLParser(Parser):
     def p_create_table(self, p):
         """create_table : CREATE TABLE IF NOT EXISTS
         | CREATE TABLE
+        | CREATE ID TABLE IF NOT EXISTS
+        | CREATE ID TABLE
 
         """
         # get schema & table name
-        pass
+        external = False
+        if p[2].upper() == "EXTERNAL":
+            external = True
+        p[0] = {"external": external}
 
     def p_column(self, p):
         """column : ID ID
-                  | column NUM_VALUE_SDP 
-                  | column ARRAY
-                  | column ID 
-                  
+        | column LP ID RP
+        | column ID
+        | column LP ID COMMA ID RP
+        | column ARRAY
+
         """
         if isinstance(p[1], dict):
             p[0] = p[1]
@@ -218,31 +306,35 @@ class DDLParser(Parser):
             size = None
             type_str = p[2]
             p[0] = {"name": p[1], "type": type_str, "size": size}
-        p_list = list(p)
-        
-        if '[]' == p_list[-1]:
-            p[0]['type'] = p[0]['type'] + '[]'
-        elif 'ARRAY' in p_list[-1]:
-            arr_split = p_list[-1].split('ARRAY')
-            append = '[]' if not arr_split[-1] else arr_split[-1]
-            p[0]['type'] = p[0]['type'] + append
+        p_list = remove_par(list(p))
+        if "[]" == p_list[-1]:
+            p[0]["type"] = p[0]["type"] + "[]"
+        elif "ARRAY" in p_list[-1]:
+            arr_split = p_list[-1].split("ARRAY")
+            append = "[]" if not arr_split[-1] else arr_split[-1]
+            p[0]["type"] = p[0]["type"] + append
         else:
             match = re.match(r"[0-9]+", p_list[2])
             if bool(match):
                 size = int(p_list[2])
-                p[0]['size'] = size
+                if len(p_list) == 3:
+                    p[0]["size"] = size
+                else:
+                    p[0]["size"] = (int(p_list[2]), int(p_list[4]))
+
     def extract_references(self, p_list):
         ref_index = p_list.index(_ref)
         if not "." in p_list[ref_index:]:
+
             references = {
                 "table": p_list[ref_index + 1],
-                "column": p_list[ref_index + 2],
+                "columns": p_list[-1],
                 "schema": None,
             }
         else:
             references = {
                 "schema": p_list[ref_index + 1],
-                "column": p_list[ref_index + 4],
+                "columns": p_list[-1],
                 "table": p_list[ref_index + 3],
             }
         return references
@@ -259,8 +351,9 @@ class DDLParser(Parser):
 
     def p_def(self, p):
         """def : DEFAULT ID
-        | DEFAULT NUM_VALUE_SDP
+        | DEFAULT STRING
         | def ID
+        | def LP RP
         """
         p_list = list(p)
         default = p[2]
@@ -268,20 +361,25 @@ class DDLParser(Parser):
             default = int(default)
         if isinstance(p[1], dict):
             p[0] = p[1]
-            if isinstance(p[2], str):
-                p[0]["default"] += f" {p[2]}"
-                p[0]["default"] = p[0]["default"].replace('"', "").replace("'", "")
+            for i in p[2:]:
+                if isinstance(p[2], str):
+                    p[2] = p[2].replace("\\'", "'")
+                    if i == ")" or i == "(":
+                        p[0]["default"] += f"{i}"
+                    else:
+                        p[0]["default"] += f" {i}"
+                    p[0]["default"] = p[0]["default"].replace('"', "").replace("'", "").replace("\\'", "'")
         else:
             p[0] = {"default": default}
 
     def p_defcolumn(self, p):
-        """expr : column
-        | expr null
-        | expr PRIMARY KEY
-        | expr UNIQUE
-        | expr check_st
-        | expr def
-        | expr ref
+        """defcolumn : column
+        | defcolumn null
+        | defcolumn PRIMARY KEY
+        | defcolumn UNIQUE
+        | defcolumn check_ex
+        | defcolumn def
+        | defcolumn ref
         """
         pk = False
         nullable = True
@@ -291,17 +389,19 @@ class DDLParser(Parser):
         references = None
         p[0] = p[1]
         p_list = list(p)
-
         if ("KEY" in p or "key" in p) and ("PRIMARY" in p or "primary" in p):
             pk = True
             nullable = False
         if "unique" in p or "UNIQUE" in p:
             unique = True
         if isinstance(p_list[-1], dict) and "references" in p_list[-1]:
+            p_list[-1]["references"]["column"] = p_list[-1]["references"]["columns"][0]
+            del p_list[-1]["references"]["columns"]
             references = p_list[-1]["references"]
         for item in p[1:]:
             if isinstance(item, dict):
                 p[0].update(item)
+
         p[0].update({"primary_key": pk, "references": references, "unique": unique})
         p[0]["nullable"] = p[0].get("nullable", nullable)
         p[0]["default"] = p[0].get("default", default)
@@ -309,8 +409,8 @@ class DDLParser(Parser):
         if p[0]["check"]:
             p[0]["check"] = " ".join(p[0]["check"])
 
-    def p_expr_check(self, p):
-        """expr :  check_st
+    def p_check_ex(self, p):
+        """check_ex :  check_st
         | constraint check_st
         """
         name = None
@@ -345,11 +445,13 @@ class DDLParser(Parser):
         p[0] = {"constraint": {"name": name}}
 
     def p_check_st(self, p):
-        """check_st : CHECK ID
+        """check_st : CHECK LP ID
         | check_st ID
-        | check_st ID ID
+        | check_st STRING
+        | check_st ID RP
+        | check_st STRING RP
         """
-        p_list = list(p)
+        p_list = remove_par(list(p))
         if isinstance(p[1], dict):
             p[0] = p[1]
         else:
@@ -366,12 +468,11 @@ class DDLParser(Parser):
             p[0].update(p[2])
 
     def p_alter_check(self, p):
-        """alter_check : alt_table CHECK ID
-        | alt_table constraint CHECK ID
-        | alter_check ID
+        """alter_check : alt_table check_st
+        | alt_table constraint check_st
         """
 
-        p_list = list(p)
+        p_list = remove_par(list(p))
         p[0] = p[1]
         if isinstance(p[1], dict):
             p[0] = p[1]
@@ -379,7 +480,30 @@ class DDLParser(Parser):
             p[0]["check"] = {"constraint_name": None, "statement": []}
         if isinstance(p[2], dict) and "constraint" in p[2]:
             p[0]["check"]["constraint_name"] = p[2]["constraint"]["name"]
-        p[0]["check"]["statement"].append(p_list[-1])
+        p[0]["check"]["statement"] = p_list[-1]["check"]
+
+    def p_pid_with_type(self, p):
+        """pid_with_type :  column
+        | pid_with_type COMMA column
+        """
+        p_list = list(p)
+        if not isinstance(p_list[1], list):
+            p[0] = [p_list[1]]
+        else:
+            p[0] = p_list[1]
+            p[0].append(p_list[-1])
+
+    def p_pid(self, p):
+        """pid :  ID
+        | pid COMMA ID
+        | STRING
+        """
+        p_list = list(p)
+        if not isinstance(p_list[1], list):
+            p[0] = [p_list[1]]
+        else:
+            p[0] = p_list[1]
+            p[0].append(p_list[-1])
 
     def p_alter_foreign(self, p):
         """alter_foreign : alt_table foreign
@@ -387,17 +511,20 @@ class DDLParser(Parser):
         """
 
         p_list = list(p)
+
         p[0] = p[1]
         if isinstance(p_list[-1], list):
-            column = {"name": p_list[-1][0]}
+            p[0]["columns"] = [{"name": i} for i in p_list[-1]]
         else:
             column = p_list[-1]
-        if isinstance(p_list[2], dict) and "constraint" in p_list[2]:
-            column.update({"constraint_name": p_list[2]["constraint"]["name"]})
 
-        if not p[0].get("columns"):
-            p[0]["columns"] = []
-        p[0]["columns"].append(column)
+            if not p[0].get("columns"):
+                p[0]["columns"] = []
+            p[0]["columns"].append(column)
+
+        for column in p[0]["columns"]:
+            if isinstance(p_list[2], dict) and "constraint" in p_list[2]:
+                column.update({"constraint_name": p_list[2]["constraint"]["name"]})
 
     def p_alt_table_name(self, p):
         """alt_table : ALTER TABLE ID ADD
@@ -415,56 +542,53 @@ class DDLParser(Parser):
 
     def p_foreign(self, p):
         # todo: need to redone id lists
-        """foreign : FOREIGN KEY ID
-        | foreign ID
-        """
-        p_list = list(p)
+        """foreign : FOREIGN KEY LP pid RP"""
+        p_list = remove_par(list(p))
         key_index = p_list.index("KEY")
-        columns = p_list[key_index + 1 :]
+        columns = p_list[-1]
 
         p[0] = columns
 
     def p_ref(self, p):
-        """ref : REFERENCES ID ID
-        | REFERENCES ID DOT ID ID
-        | ref ID
+        """ref : REFERENCES ID LP pid RP
+        | REFERENCES ID DOT ID LP pid RP
         """
-        p_list = list(p)
-        if isinstance(p[1], dict):
-            p[0] = p[1]
-            for column in p_list[2:]:
-                p[0]["references"]["column"].append(column)
-        else:
-            data = {"references": self.extract_references(p_list)}
-            p[0] = data
+        p_list = remove_par(list(p))
+        data = {"references": self.extract_references(p_list)}
+        p[0] = data
 
     def p_expression_primary_key(self, p):
         "expr : pkey"
         p[0] = p[1]
 
-    def p_expression_uniq(self, p):
-        "expr : uniq"
-        p[0] = p[1]
-
     def p_uniq(self, p):
-        """uniq : UNIQUE ID
+        """uniq : UNIQUE LP ID
+        | uniq COMMA
         | uniq ID
+        | uniq RP
+
         """
+        p_list = list(p)
         if isinstance(p[1], dict):
             p[0] = p[1]
-            p[0]["unique"].append(p[2])
+            p[0]["unique_statement"].append(p_list[-1])
         else:
-            p[0] = {"unique": [x for x in p[1:] if x != ","]}
+            p[0] = {"unique_statement": [x for x in p[2:] if x != ","]}
 
     def p_pkey(self, p):
-        """pkey : PRIMARY KEY ID
-        | pkey ID
-        """
-        if isinstance(p[1], dict):
-            p[0] = p[1]
-            p[0]["primary_key"].append(p[2])
-        else:
-            p[0] = {"primary_key": [x for x in p[3:] if x != ","]}
+        """pkey : PRIMARY KEY LP pid RP"""
+        p_list = remove_par(list(p))
+        p[0] = {"primary_key": p_list[-1]}
+
+
+def remove_par(p_list):
+    if "(" in p_list:
+        p_in = p_list.index("(")
+        p_list.pop(p_in)
+    if ")" in p_list:
+        p_in = p_list.index(")")
+        p_list.pop(p_in)
+    return p_list
 
 
 def parse_from_file(file_path: str, **kwargs) -> List[Dict]:
