@@ -3,23 +3,25 @@ import json
 from typing import Dict, List, Union, Tuple
 
 
-def get_table_from_tables_data(tables_dict: Dict, table_id: Tuple[str, str], statement: Dict) -> Dict:
-    
+def get_table_from_tables_data(
+    tables_dict: Dict, table_id: Tuple[str, str], statement: Dict
+) -> Dict:
+
     target_table = tables_dict.get(table_id)
     if target_table is None:
         raise ValueError(
-            f"Found ALTER statement to not existed TABLE {statement['alter_table_name']} with SCHEMA {statement['schema']}"
+            f"Found ALTER statement to not existed TABLE {table_id[0]} with SCHEMA {table_id[1]}"
         )
     return target_table
 
 
 def add_index_to_table(tables_dict: Dict, statement: Dict) -> Dict:
-    
+
     table_id = (statement["table_name"], statement["schema"])
     target_table = get_table_from_tables_data(tables_dict, table_id, statement)
-    del statement['schema']
-    del statement['table_name']
-    target_table['index'].append(statement)
+    del statement["schema"]
+    del statement["table_name"]
+    target_table["index"].append(statement)
     return tables_dict
 
 
@@ -29,10 +31,7 @@ def add_alter_to_table(tables_dict: Dict, statement: Dict) -> Dict:
     if "columns" in statement:
         alter_columns = []
         for num, column in enumerate(statement["columns"]):
-
-            if isinstance(statement["references"]["column"], str):
-                statement["references"]["column"] = [statement["references"]["column"]]
-            column_reference = statement["references"]["column"][num]
+            column_reference = statement["references"]["columns"][num]
             alter_column = {
                 "name": column["name"],
                 "constraint_name": column.get("constraint_name"),
@@ -61,11 +60,20 @@ def set_checks_to_table(table_data: Dict, check: Union[List, Dict]) -> Dict:
     return table_data
 
 
-def result_format(result: List[Dict]) -> List[Dict]:
+def result_format(result: List[Dict], output_mode: str) -> List[Dict]:
     final_result = []
     tables_dict = {}
     for table in result:
-        table_data = {"columns": [], "primary_key": None, "alter": {}, "checks": [], "index": []}
+        table_data = {
+            "columns": [],
+            "primary_key": None,
+            "alter": {},
+            "checks": [],
+            "index": [],
+            "partitioned_by": [],
+        }
+        if output_mode == "hql":
+            table_data = add_additional_hql_keys(table_data)
         sequence = False
         if len(table) == 1 and "index_name" in table[0]:
             tables_dict = add_index_to_table(tables_dict, table[0])
@@ -78,16 +86,13 @@ def result_format(result: List[Dict]) -> List[Dict]:
                     sequence = True
                     continue
                 elif item.get("table_name"):
-                    table_data["table_name"] = item["table_name"]
-                    table_data["schema"] = item["schema"]
-                elif not item.get("type") and item.get("primary_key"):
-                    table_data["primary_key"] = item["primary_key"]
-                elif not item.get("type") and item.get("unique"):
-                    table_data["unique"] = item["unique"]
-                elif not item.get("type") and item.get("check"):
-                    table_data = set_checks_to_table(table_data, item["check"])
-                else:
-                    table_data["columns"].append(item)
+                    table_data.update(item)
+                    if "unique_statement" in table_data:
+                        for column in table_data["columns"]:
+                            if column["name"] in table_data["unique_statement"]:
+                                column["unique"] = True
+                        del table_data["unique_statement"]
+
             if not sequence:
                 if table_data.get("table_name"):
                     tables_dict[
@@ -108,10 +113,47 @@ def result_format(result: List[Dict]) -> List[Dict]:
                 for column in table_data["columns"]:
                     if column["name"] in table_data["primary_key"]:
                         column["nullable"] = False
+            if output_mode != "hql":
+                table_data = clean_up_output(table_data)
+            else:
+                # todo: need to figure out how workaround it normally
+                if "_ddl_parser_comma_only_str" == table_data["fields_terminated_by"]:
+                    table_data["fields_terminated_by"] = ","
             final_result.append(table_data)
     return final_result
 
 
+def add_additional_hql_keys(table_data: Dict) -> Dict:
+    table_data.update(
+        {
+            "stored_as": None,
+            "location": None,
+            "row_format": None,
+            "fields_terminated_by": None,
+            'map_keys_terminated_by': None,
+            'collection_items_terminated_by': None
+        }
+    )
+    return table_data
+
+
+def clean_up_output(table_data: Dict) -> Dict:
+    key_list = [
+        "external",
+        "external",
+        "stored_as",
+        "location",
+        "row_format",
+        "fields_terminated_by",
+        "collection_items_terminated_by",
+        "map_keys_terminated_by",
+    ]
+    for key in key_list:
+        if key in table_data:
+            del table_data[key]
+    return table_data
+
+
 def add_unique_columns(table_data: Dict) -> Dict:
     for column in table_data["columns"]:
         if column["name"] in table_data["unique"]:
